#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>
#include <ctype.h>

#define MAXLINE 5120
#define MAXNUM  100

int loadCourse(char course[MAXNUM][MAXNUM][MAXNUM]);
void process(int server_sock_data, char course[MAXNUM][MAXNUM][MAXNUM], int stu_num);
int Socket(int family, int type, int protocol);
int Inet_aton(const char *strptr, struct in_addr *addrptr);
int Bind(int sockfd, const struct sockaddr *addr, socklen_t addrlen);
int Connect(int sockfd, const struct sockaddr *servaddr, socklen_t addrlen);
int Listen(int sockfd, int backlog);
int Accept(int sockfd, struct sockaddr *addr, socklen_t *addrlen);
ssize_t Send(int sockfd, const void *buf, size_t len, int flags);
ssize_t Recv(int sockfd, void *buf, size_t len, int flags);
int Close(int fd);

int main(int argc, char *argv[])
{
	int stu_num;
	int server_sock_listen, server_sock_data;
	struct sockaddr_in server_addr;
	char course[MAXNUM][MAXNUM][MAXNUM];

	if (argc != 2) {
		printf("Usage: %s port\n", argv[0]);
		exit(1);
	}

	// 读取课程表
	stu_num = loadCourse(course);

	/* 创建socket */
	server_sock_listen = Socket(AF_INET, SOCK_STREAM, 0);

	/* 指定服务器地址 */
	server_addr.sin_family = AF_INET;
	server_addr.sin_port = htons(atoi(argv[1]));
	server_addr.sin_addr.s_addr = htonl(INADDR_ANY); //INADDR_ANY表示本机所有IP地址
	memset(&server_addr.sin_zero, 0, sizeof(server_addr.sin_zero)); //零填充

	/* 绑定socket与地址 */
	Bind(server_sock_listen, (struct sockaddr *)&server_addr, sizeof(server_addr));

	/* 监听socket */
	Listen(server_sock_listen, 0);

	while (1) {
		printf("Server is listening....\n");
		server_sock_data = Accept(server_sock_listen, NULL, NULL);  //接受客户端连接
		printf("Accept a client\n");

		process(server_sock_data, course, stu_num);
	}

	/* 关闭监听socket */
	Close(server_sock_listen);

	return 0;
}

void process(int server_sock_data, char course[MAXNUM][MAXNUM][MAXNUM], int stu_num) {
	char recv_msg[MAXLINE];
	char send_msg[MAXLINE];
	while (1) {
		/* 接收并显示消息 */
		memset(recv_msg, 0, sizeof(recv_msg)); //接收数组置零
		if (Recv(server_sock_data, recv_msg, sizeof(recv_msg), 0) == 0) {	// 如果客户端关闭了连接就退出
			printf("Bye!\n\n");
			break;
		}
		printf("client: %s", recv_msg);
		if (strcasecmp(recv_msg, "bye\n") == 0) {							// 如果客户端输入的是 bye 那么就退出
			printf("Bye!\n\n");
			break;
		}

		int len = strlen(recv_msg);
		if (recv_msg[len - 1] == '\n')
			recv_msg[len - 1] = '\0'; //去掉换行符

		char *sp = strchr(recv_msg, ','), *id = recv_msg;
		int num;
		if (sp == NULL)	// 如果没有逗号，说明要输出所有课程，相当于是课程号为 0
			num = 0;
		else {
			*sp = '\0';
			char *nums = sp + 1;
			if (*id == '\0') {
				strcpy(send_msg, "输入错误：请输入学号\n");
				goto send;
			}
			if (*nums == '\0') {
				strcpy(send_msg, "输入错误：请输入课程号\n");
				goto send;
			}
			for (char *p = nums; *p != '\0'; ++p)
				if (!isdigit(*p)) {
					strcpy(send_msg, "输入错误：请输入整数课程号\n");
					goto send;
				}
			num = atoi(nums);
		}
		// 现在我们的 id 中是学号，num 中是课程号
		char (*stu)[MAXNUM] = NULL;
		for (int i = 0; i < stu_num; ++i)
			if (strcmp(course[i][0], id) == 0) {
				stu = course[i];
				break;
			}
		// fprintf(stderr, "**************** %p\n", stu);
		// 现在，stu 是我们找到的学生的信息对应的数组
		if (stu == NULL) {
			strcpy(send_msg, "输入错误：找不到该学生\n");
			goto send;;
		}

		char *ptr = send_msg;		// ptr 是用来指示下面应该在 send_msg 字符串的哪里继续添加的
		ptr += sprintf(ptr, "%s %s", stu[0], stu[1]);
		// fprintf(stderr, "****** %d %d\n", num, coursenum[stu_num]);
		int flag = 0;
		for (int i = 2; stu[i][0]; ++i)
			if (!num || i == num + 1) {	// 如果课程号为 0，输出所有课程，否则输出询问的那一个课程
				ptr += sprintf(ptr, " %s", stu[i]);
				flag = 1;
			}
		if (flag == 0) {
			strcpy(send_msg, "输入错误：课程序号错误\n");
			goto send;
		}
		ptr += sprintf(ptr, "\n");

	    send:
		/* 发送消息 */
		printf("server: %s", send_msg);
		Send(server_sock_data, send_msg, strlen(send_msg), 0);
	}

	/* 关闭数据socket */
	Close(server_sock_data);
}

int loadCourse(char course[MAXNUM][MAXNUM][MAXNUM]) {
	char buf[MAXLINE];
	int stu_num;
	FILE *fp = fopen("课程表.txt", "r");
	while (fgets(buf, MAXLINE, fp) != NULL) {
		// fputs(buf, stdout);
		char *ptr = buf;
		for (int i = 0; ; ++i) {
			int re, n;
			re = sscanf(ptr, "%s%n", course[stu_num][i], &n);
			ptr += n;
			if (re == EOF) {
				course[stu_num][i][0] = '\0';
				break;
			}
		}
		++stu_num;
	}
	fclose(fp);
	return stu_num;
}

int Socket(int family, int type, int protocol) {
	int n;
	if ((n = socket(family, type, protocol)) < 0) {
		perror("socket error");
		exit(1);
	}
	return n;
}
int Inet_aton(const char *strptr, struct in_addr *addrptr) {
	int n;
	if ((n = inet_aton(strptr, addrptr)) == 0) {
		perror("inet_aton error");
		exit(1);
	}
	return n;
}
int Bind(int sockfd, const struct sockaddr *addr, socklen_t addrlen) {
	int n;
	if ((n = bind(sockfd, addr, addrlen)) < 0) {
		perror("bind error");
		exit(1);
	}
	return n;
}
int Connect(int sockfd, const struct sockaddr *servaddr, socklen_t addrlen) {
	int n;
	if ((n = connect(sockfd, servaddr, addrlen)) < 0) {
		perror("connect error");
		exit(1);
	}
	return n;
}
int Listen(int sockfd, int backlog) {
	int n;
	if ((n = listen(sockfd, backlog)) < 0) {
		perror("listen error");
		exit(1);
	}
	return n;
}
int Accept(int sockfd, struct sockaddr *addr, socklen_t *addrlen) {
	int n;
	if ((n = accept(sockfd, addr, addrlen)) < 0) {
		perror("accept error");
		exit(1);
	}
	return n;
}
ssize_t Send(int sockfd, const void *buf, size_t len, int flags) {
	ssize_t n;
	if ((n = send(sockfd, buf, len, flags)) < 0) {
		perror("send error");
		exit(1);
	}
	return n;
}
ssize_t Recv(int sockfd, void *buf, size_t len, int flags) {
	ssize_t n;
	if ((n = recv(sockfd, buf, len, flags)) < 0) {
		perror("recv error");
		exit(1);
	}
	return n;
}
int Close(int fd) {
	int n;
	if ((n = close(fd)) < 0) {
		perror("close error");
		exit(1);
	}
	return n;
}